;**********************************************************************
;                                                                     *
;    Filename:	    mtx-013.asm                                       *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:        el@jap.hu                                         *
;                   http://jap.hu/                                    *
;**********************************************************************
;    Notes:
;
;RF transmitter prot.v4 (Manchester)
;supposed crystal freq is 4MHz
;frequency is 400/800 Hz
; delay times = 625 usec, 625 usec
;**********************************************************************
;v2 HISTORY
;
;002-19991227
;low power standby mode, no POWER switch needed
;transmit portb4-7 buttons state if any is pressed
;sleep until a change occurs on the port
;
;003-19991227
;code is compilable for both the F84 and the C508
;call depth is reduced to 2 (hopefully every place)
;this is the working version!
;
;004-19991228
;experimental version only sending bit patterns by itself
;
;005-20000316
;no sleep, 12c509a
;
;**********************************************************************
;v3 HISTORY
;
;006-20001223
;new prot. v3: less bytes to transmit (-5) & at lower speed
;wake-up logic enabled
;
; header: 8xbit0, 1xbit1
; devid : 2xbyte
; data  : 1xbyte
; chksum: 1xbyte
;
;007-20001225
;one-button version, transmit number of times the button is pressed
;transmit a signal three times
;
;008-20001225
;same as 007, except a new DEFINE TXINV invert TX output
;
;009-20001226
;new output for ASK modulation
;
;010-20001227
;adaption for the 12c509a
;
;011-20010109
;manchester coded output
;
;012-20010113
;continous modulation out (output 1-bits while idle)
;one button only, planning 5 for the final version
;
; header : 8xbit1, 1xbit0
; bytes : 8xbitX, 1xbit1, 1xbit0
;
;013-20010113
;packet struct: +1 byte, new devid: 9ae6
;should send 5 button states
;
;**********************************************************************

;define F84
#define C508
;define TXINV
;invert TX output so inactive is HIGH?

#ifdef F84
	list      p=16F84             ; list directive to define processor
	#include <p16F84.inc>         ; processor specific variable definitions

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC
	;WDT=watchdog, PWRTE=power on timer, XT= crystal osc

freemem EQU 0x10
txbit	EQU 1
keymask EQU 0xf0
ioport	EQU PORTB

#endif
#ifdef C508
	list	p=12c508
	#include <p12C508.inc>

	__CONFIG   _CP_OFF & _WDT_OFF & _MCLRE_OFF & _IntRC_OSC

freemem EQU 0x07
txbit	EQU 0x20
keymask	EQU 0x1f
ioport	EQU GPIO

#endif

#ifndef TXINV
#define TXLOW 0
#define TXHIGH txbit
#else
#define TXLOW txbit
#define TXHIGH 0
#endif

;***** VARIABLE DEFINITIONS

count1		EQU	freemem
count2		EQU	freemem+1
ncnt		EQU	freemem+2 ; ndelay counter
bt		EQU	freemem+3
sum		EQU	freemem+4
buffer		EQU	freemem+5
bdevid		EQU	buffer  ; 2 byte DEVID
bdata		EQU	buffer+2; 'real' data

delayconst	EQU	.202 ; ~625 usec @4MHz, half_frame delay

;**********************************************************************
#ifdef F84
		ORG     0x000             ; processor reset vector
  		goto    main              ; go to beginning of program
		nop
		nop
		nop

		;ORG     0x004             ; interrupt vector location
		retfie
#endif

#ifdef C508
		ORG	0x000
		movwf	OSCCAL
		;C508 HARDWARE INIT
		clrf	GPIO ; clear all I/O pins
		movlw	keymask ;keys are IN, TX out
		TRIS	GPIO ; I/O directions

		movlw	0
		OPTION
		;-GPWU wakeup ENA
		;-GPPU pullup ENA on GP0,1,3
		;T0CS disable: enablings implicitly sets GP2 dir as input!

		goto	main
#endif

table		addwf PCL, F
devid		dt 0x9a, 0xe6, 0, 0x5a

main		
#ifdef F84
		;F84 HARDWARE INIT
		clrf PORTA
		clrf PORTB

		MOVLW keymask; PB0 output, 4-7 IN
		TRIS PORTB
		MOVLW 0; PA all out, not used
		TRIS PORTA

		movlw 0x3
		OPTION ; pres 1:16, B4-B7 PU ena
#endif
		call readbuf
		;
warm		; output 11 header bits to quit false rxbuf
		movlw 0x0b ; 11xbit1
		movwf count2
w_head		call bit1
		decfsz count2,F
		goto w_head

loop		; check if a button is pressed
		movlw keymask
		andwf ioport, W
		movwf bdata ; store key data in buffer

		xorlw keymask ; test for H bits
		bnz keys_pressed

no_keys		; countinously output a bit1
		call bit1
		goto loop

keys_pressed	; at least one key is pressed in this cycle
		incf bdata+1, F

outbuf		movlw 0x10 ; 16xbit1, 1xbit0

header		movwf count2
head0		call bit1
		decfsz count2,F
		goto head0
		call bit0

		movlw buffer
		movwf FSR
		movlw 0x04
		movwf count1
		clrf sum
		;
outbu0		movf INDF,W
		addwf sum,F
		movf INDF,W
		call outbyte
		incf FSR,F
		decfsz count1,F
		goto outbu0
		movf sum,W
		call outbyte
		; buffer is sent

		goto warm
		;

outbyte		movwf bt
		movlw 8
		movwf count2
outby0		rlf bt,F
		btfsc STATUS,C
		goto outby1
		call bit0
		goto outby2
outby1		call bit1
outby2		decfsz count2,F
		goto outby0
		;
		call bit1
		; and bit0

bit0		movlw TXHIGH ; HIGH
		movwf ioport

ndelaya0	movlw delayconst
		movwf ncnt
ndelaya1	decfsz ncnt, F
		goto ndelaya1

		movlw TXLOW ; to LOW transition
		movwf ioport

ndelayb0	movlw delayconst
		movwf ncnt
ndelayb1	decfsz ncnt, F
		goto ndelayb1

		return

bit1		movlw TXLOW ; LOW
		movwf ioport

ndelayc0	movlw delayconst
		movwf ncnt
ndelayc1	decfsz ncnt, F
		goto ndelayc1

		movlw TXHIGH ; to HIGH transition
		movwf ioport

ndelaye0	movlw delayconst
		movwf ncnt
ndelaye1	decfsz ncnt, F
		goto ndelaye1

		return

readbuf		;read data into buffer
		movlw buffer
		movwf FSR
		clrf count1
readbu0		movf count1, W
		call table
		movwf INDF
		incf FSR,F
		incf count1,F

		movlw 0x04
		subwf count1, W
		bnz readbu0

		return

		end

